
// Copyright (c) 1996-2003 The University of Cincinnati.  
// All rights reserved.

// UC MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
// SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
// OR NON-INFRINGEMENT.  UC SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
// LICENSEE AS A RESULT OF USING, RESULT OF USING, MODIFYING OR
// DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

// By using or copying this Software, Licensee agrees to abide by the
// intellectual property laws, and all other applicable laws of the U.S.,
// and the terms of this license.

// You may modify, distribute, and use the software contained in this
// package under the terms of the "GNU LIBRARY GENERAL PUBLIC LICENSE"
// version 2, June 1991. A copy of this license agreement can be found in
// the file "LGPL", distributed with this archive.

// Authors: Philip A. Wilsey	philip.wilsey@ieee.org
//          Dale E. Martin	dmartin@cliftonlabs.com
//          Timothy J. McBrayer
//          Malolan Chetlur    

//---------------------------------------------------------------------------

#include "IIR_MultiplicationOperator.hh"
#include "set.hh"
#include "error_func.hh"
#include "resolution_func.hh"
#include "error_func.hh"
#include "IIR_TypeDefinition.hh"
#include "sstream-wrap.hh"
#include "published_file.hh"

IIRScram_MultiplicationOperator::~IIRScram_MultiplicationOperator() {}

void 
IIRScram_MultiplicationOperator::_publish_vhdl_operator(ostream &_vhdl_out) {
  _vhdl_out << " * ";
}

const string &
IIRScram_MultiplicationOperator::_get_cc_operator_name() const {
  static const string operator_name = "vhdlMultiply";
  return operator_name;
}

void
IIRScram_MultiplicationOperator::_publish_cc_ams_operator_name( published_file &_cc_out ) {
  _cc_out << " * ";
}

IIR*
IIRScram_MultiplicationOperator::_clone() {
  IIR_MultiplicationOperator *clone = new IIR_MultiplicationOperator();
  IIRScram_DyadicOperator::_clone(clone);
  return clone;
}

IIRScram::OperatorPrecedenceLevel
IIRScram_MultiplicationOperator::_get_operator_precedence(){
  return IIRScram::MULTIPLYING_OPERATOR;
}

bool
IIRScram_MultiplicationOperator::_is_associative(IIR_Kind oper_kind){
  switch(oper_kind) {                         
  case IIR_REMAINDER_OPERATOR:                
  case IIR_MODULUS_OPERATOR:                
    return FALSE;                             
  default:
    return TRUE;                                                             
  }

}

visitor_return_type *IIRScram_MultiplicationOperator::_accept_visitor(node_visitor *visitor, visitor_argument_type *arg) {
  ASSERT(visitor != NULL);
  return visitor->visit_IIR_MultiplicationOperator(this, arg);
};
