// Copyright (c) 1996-2000 The University of Cincinnati.  
// All rights reserved.

// UC MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  UC SHALL NOT BE LIABLE
// FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
// RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
// DERIVATIVES.

// By using or copying this Software, Licensee agrees to abide by the
// intellectual property laws, and all other applicable laws of the
// U.S., and the terms of this license.


// You may modify, distribute, and use the software contained in this package
// under the terms of the "GNU LIBRARY GENERAL PUBLIC LICENSE" version 2,
// June 1991. A copy of this license agreement can be found in the file
// "LGPL", distributed with this archive.

// Authors: Philip A. Wilsey	phil.wilsey@uc.edu
//          Dale E. Martin	dmartin@ece.uc.edu
//          Timothy J. McBrayer tmcbraye@ece.uc.edu
//          Malolan Chetlur     mal@ece.uc.edu
//          Krishnan Subramani  skrish@ece.uc.edu

//---------------------------------------------------------------------------
#include "IIR_NorOperator.hh"
#include "IIR_TypeDefinition.hh"
#include "savant.hh"
#include "published_file.hh"

IIRScram_NorOperator::~IIRScram_NorOperator() {}

void 
IIRScram_NorOperator::_publish_vhdl_operator(ostream &_vhdl_out) {
  _vhdl_out << " nor ";
}

const string &
IIRScram_NorOperator::_get_cc_operator_name() const {
  static const string operator_name = "vhdlNor";
  return operator_name;
}

IIR*
IIRScram_NorOperator::_clone() {
  IIR_NorOperator *clone = new IIR_NorOperator();
  IIRScram_DyadicOperator::_clone(clone);
  return clone;
}

IIRScram::OperatorPrecedenceLevel
IIRScram_NorOperator::_get_operator_precedence(){
  return IIRScram::LOGICAL_OPERATOR;
}

void
IIRScram_NorOperator::_publish_cc_xvalue( published_file &_cc_out,
					 void (IIR::*_publish_function)( published_file & )) {
  if( get_subtype()->_is_bit_type() ||
      get_subtype()->_is_boolean_type() ){
    _publish_cc_short_circuit_negative( _cc_out, _publish_function );
  }
  else{
    IIR_DyadicOperator::_publish_cc_xvalue( _cc_out, _publish_function );
  }
}

bool
IIRScram_NorOperator::_is_associative(IIR_Kind oper_kind){
  switch(oper_kind) {                                                        
  case IIR_OR_OPERATOR:
  case IIR_NAND_OPERATOR:
  case IIR_AND_OPERATOR:
  case IIR_XOR_OPERATOR:
  case IIR_XNOR_OPERATOR:
  case IIR_NOR_OPERATOR:
    return FALSE;                                                             
  default:
    return TRUE;                                                             
  }                  
}

IIR_Boolean
IIRScram_NorOperator::_is_left_associative(){
  return FALSE;
}

IIR_Boolean 
IIRScram_NorOperator::_is_logical_operator(){
  return TRUE ;
}

visitor_return_type *
IIRScram_NorOperator::_accept_visitor( node_visitor *visitor, 
				       visitor_argument_type *arg ){
  ASSERT(visitor != NULL);
  return visitor->visit_IIR_NorOperator(this, arg);
};
