// Copyright (c) 1996-2000 The University of Cincinnati.  
// All rights reserved.

// UC MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  UC SHALL NOT BE LIABLE
// FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
// RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
// DERIVATIVES.

// By using or copying this Software, Licensee agrees to abide by the
// intellectual property laws, and all other applicable laws of the
// U.S., and the terms of this license.


// You may modify, distribute, and use the software contained in this package
// under the terms of the "GNU LIBRARY GENERAL PUBLIC LICENSE" version 2,
// June 1991. A copy of this license agreement can be found in the file
// "LGPL", distributed with this archive.

// Authors: Philip A. Wilsey	phil.wilsey@uc.edu
//          Dale E. Martin	dmartin@ece.uc.edu
//          Timothy J. McBrayer tmcbraye@ece.uc.edu
//          Malolan Chetlur     mal@ece.uc.edu

//---------------------------------------------------------------------------
#include "IIR_RemainderOperator.hh"
#include "resolution_func.hh"
#include "error_func.hh"
#include "set.hh"
#include "savant.hh"
#include "published_file.hh"

IIRScram_RemainderOperator::~IIRScram_RemainderOperator() {}

void 
IIRScram_RemainderOperator::_publish_vhdl_operator(ostream &_vhdl_out) {
  _vhdl_out << " rem ";
}

const string &
IIRScram_RemainderOperator::_get_cc_operator_name() const {
  static const string operator_name = "vhdlRem";
  return operator_name;
}

IIR*
IIRScram_RemainderOperator::_clone() {
  IIR_RemainderOperator *clone = new IIR_RemainderOperator();
  IIRScram_DyadicOperator::_clone(clone);
  return clone;
}

IIRScram::OperatorPrecedenceLevel
IIRScram_RemainderOperator::_get_operator_precedence(){
  return IIRScram::MULTIPLYING_OPERATOR;
}

bool
IIRScram_RemainderOperator::_is_associative(IIR_Kind oper_kind){
  switch(oper_kind){
  case IIR_MULTIPLICATION_OPERATOR:
  case IIR_DIVISION_OPERATOR:
  case IIR_REMAINDER_OPERATOR:
  case IIR_MODULUS_OPERATOR:
    return FALSE;
  default:
    return TRUE;
  }
}

visitor_return_type *
IIRScram_RemainderOperator::_accept_visitor( node_visitor *visitor, 
					     visitor_argument_type *arg ){
  ASSERT(visitor != NULL);
  return visitor->visit_IIR_RemainderOperator(this, arg);
}
